// This example is from the book _Java in a Nutshell_ by David Flanagan.
// Written by David Flanagan.  Copyright (c) 1996 O'Reilly & Associates.
// You may study, use, modify, and distribute this example for any purpose.
// This example is provided WITHOUT WARRANTY either expressed or implied.

/** This file has been modified extensively by Steve Kemp, for inclusion 
    with the java Shell.
     The main modification allows the Grep class to be used with wildcards
    if the file specified is a wildcard all the files that match it are 
    grepped, and the results of the grep is prepended with the name of the
    file, this is identical to the unix grep command.

     If only one match is made then the filename is ignored.

*/
/**
 * more modifications by Godmar Back <gback@cs.utah.edu>
 */

import java.io.*;
import java.util.*;


// This class demonstrates the use of the GrepInputStream class.
// It prints the lines of a file that contain a specified substring.
public class grep
{

    class GrepInputStream extends FilterInputStream {
	String substring;
	DataInputStream in;
    
	public GrepInputStream(DataInputStream in, String substring) {
	    super(in);
	    this.in = in;
	    this.substring = substring;
	}
	
	// This is the filter:  read lines from the DataInputStream,
	// but only return the lines that contain the substring.
	// When the DataInputStream returns null, we return null.
	public final String readLine() throws IOException {
	    String line;
	    do {
		line = in.readLine();
	    }
	    while ((line != null) && line.indexOf(substring) == -1);
	    return line;
	}
    }

  private static boolean DEBUG = false;
  private static File pwd;
  private static String currentDir;

 public static void main(String args[])
 {
   new grep(args);
 }

 grep(String args[])
 {
   if ((args.length == 0) || (args.length > 2))
   { System.out.println("Usage: java Grep <substring> [<filename>]");
   }
   else
   { 
     currentDir = new String ( System.getProperty( "user.dir" ) );

     pwd = new File( currentDir );

     try
     { DataInputStream d;
            
       if (args.length == 2) 
       { String files[] = getWildCardMatches( args[ 1 ] );
         
         for ( int i = 0; i < files.length; i++ )
         { d = new DataInputStream(new FileInputStream( files[ i ].trim() ));
           
           File f = new File( files[ i ].trim() );
           String name = f.getName();
   
           GrepInputStream g = new GrepInputStream(d, args[0]);
            
           String line;
           while ( true )
           { line = g.readLine();
             if (line == null)
                break;
             if ( files.length > 1 )
               System.out.println( name + ": " + line );
             else
               System.out.println( line );
           }
         } 
       }
       else
       { d = new DataInputStream(System.in);
            
         GrepInputStream g = new GrepInputStream(d, args[0]);
             
         String line;
         while( true )
         { line = g.readLine();
           if (line == null) break;
           System.out.println(line);
         }
         g.close();
       }
     }
     catch (IOException e)
     { System.err.println(e);
     }
   }
 }


/** This is an important method that returns an array of filenames, that
   match a given wildcard specification.
   @param s The wild card of files to allow
   @return An array containing filenames of matching files.
 */
   private static String[] getWildCardMatches( String s )
   { String args = new String( s.trim() );
     String files[];
     Vector filesThatMatchVector = new Vector();
     String filesThatMatch[];

// Get all the files in this directory
     files = pwd.list();

// Go through them, and see if they match the wildcard specified
     for ( int i = 0; i < files.length; i++ )
     { if ( match( args, files[i] ) )
       { File temp = new File( pwd, files[ i ] );
          filesThatMatchVector.addElement( new String(
            temp.toString() ) );
       }    
     }
     if ( DEBUG )
     { System.out.println( "Testing wildcard '" + args + "'" );
       System.out.println( "Found " + filesThatMatchVector.size() +
         " matches" );
     }
     filesThatMatch = new String[ filesThatMatchVector.size() ];
     filesThatMatchVector.copyInto( filesThatMatch );
     if ( DEBUG )
     { System.out.println( "Files that match" );
       for ( int i=0; i < filesThatMatch.length; i++ )
          System.out.println( "\t" + filesThatMatch[ i ] );
     }
 
     sortStrings( filesThatMatch );

     return filesThatMatch;
   }




    /// Checks whether a string matches a given wildcard pattern.
    // Only does ? and *, and multiple patterns separated by |.
    public static boolean match( String pattern, String string )
	{
	for ( int p = 0; ; ++p )
	    {
	    for ( int s = 0; ; ++p, ++s )
		{
		boolean sEnd = ( s >= string.length() );
		boolean pEnd = ( p >= pattern.length() ||
				 pattern.charAt( p ) == '|' );
		if ( sEnd && pEnd )
		    return true;
		if ( sEnd || pEnd )
		    break;
		if ( pattern.charAt( p ) == '?' )
		    continue;
		if ( pattern.charAt( p ) == '*' )
		    {
		    int i;
		    ++p;
		    for ( i = string.length(); i >= s; --i )
			if ( match(
			       pattern.substring( p ),
			       string.substring( i ) ) )  /* not quite right */
			    return true;
		    break;
		    }
		if ( pattern.charAt( p ) != string.charAt( s ) )
		    break;
		}
	    p = pattern.indexOf( '|', p );
	    if ( p == -1 )
		return false;
	    }
	}



    /// Sorts an array of Strings.
    // Java currently has no general sort function.  Sorting Strings is
    // common enough that it's worth making a special case.
    public static void sortStrings( String[] strings )
	{
	// Just does a bubblesort.
	for ( int i = 0; i < strings.length - 1; ++i )
	    {
	    for ( int j = i + 1; j < strings.length; ++j )
		{
		if ( strings[i].compareTo( strings[j] ) > 0 )
		    {
		    String t = strings[i];
		    strings[i] = strings[j];
		    strings[j] = t;
		    }
		}
	    }
	}
    

}
