package minishell;
import java.lang.reflect.*;
import java.util.Properties;
import java.io.*;
 
public class Application
{
    public static boolean debug = false;
    void log(String msg)
    {
	if (!debug)
	    return;
	System.err.println("Application: " + appname + ":" + msg);
	System.err.flush();
    }

    private String argv[];
    private String appname;
    private PrintStream errstream = System.err;

    // a name space
    private ClassPathLoader apploader;

    private boolean exited = false;
    private int status;		// exit status

    public Application(
	String argv[], 
	String classpath) throws Exception
    {
	this.argv = argv;
	this.appname = argv[0];
	this.apploader = ClassPathLoader.createClassLoader(classpath);
    }

    private void invokeStaticMethod(Method m, Object a) throws Exception
    {
	Object [] o = new Object[] { a };
	log("invoke " + m + " with args " + o[0]);
	m.invoke(null /* static method */, a == null ? null : o);
    }

    public void run()
    {
	try {
	    log("Loading class " + appname);
	    Class c = apploader.loadClass(appname, true);

	    log("find main method");
	    Method main = c.getMethod("main", 
		new Class[] { java.lang.String[].class });

	    String args[] = new String[argv.length - 1];
	    System.arraycopy(argv, 1, args, 0, args.length);
	    invokeStaticMethod(main, args);

	} catch (ClassNotFoundException e) {
	    errstream.println("Error: Class " 
		+ appname + " not found.");
	} catch (NoSuchMethodException e) {
	    errstream.println("Error: " 
		+ appname + " does not have a main method.");
	} catch (InvocationTargetException e) {
	    Throwable t = e.getTargetException();

	    // don't say anything if thread was killed
	    if (!(t instanceof ThreadDeath)) {
		errstream.println("Error: application " + appname 
		    + " threw the following exception:\n");
		t.printStackTrace(errstream);
	    }
	} catch (Throwable e) {
	    errstream.println("Unknown exception occurred: ");
	    e.printStackTrace(errstream);
	}
    }

    public String toString()
    {
	return "application=" + appname;
    }
}
